package com.ceb.gjjs.mda.web;

import com.brilliance.mda.runtime.mda.IAuthInfo;
import com.brilliance.mda.runtime.mda.IContext;
import com.brilliance.mda.runtime.mda.driver.MdaEnv;
import com.brilliance.mda.runtime.mda.util.Systems;
import com.brilliance.mda.runtime.request.BaseVO;
import com.brilliance.mda.support.cache.ICache;
import com.brilliance.mda.support.td.TDAuthInfo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.Date;

import static com.brilliance.mda.runtime.mda.Constants.tdContextLASTUSERACTIONTIME;

public class AuthInterceptor implements HandlerInterceptor {

    @Qualifier(ICache.CTX_CACHE)
    @Autowired
    private ICache<String, IContext> contextICache;

    @Qualifier(ICache.AUTH_CACHE)
    @Autowired
    private ICache<String, IAuthInfo> authInfoICache;

    //简单生成Token Key
    private String getTokenKey(HttpServletRequest request){
        String logName = request.getHeader("token");
        if(logName == null || "undefined".equals(logName))
            logName = "sun123";
        //现在仅仅以userId这个来实现
        return logName;
    }

    private IAuthInfo getOrCreateAuthInfo(String tokenKey,String userId){
        IAuthInfo authInfo = authInfoICache.get(tokenKey);
        if (authInfo == null) {
            synchronized (authInfoICache) {
                authInfo = authInfoICache.get(tokenKey);
                if(authInfo == null){
                    TDAuthInfo tdAuthInfo = new TDAuthInfo(userId);
                    authInfo = tdAuthInfo;
                    authInfoICache.store(tokenKey,authInfo);
                    //构造authInfo，本该在登录里面做
                  //  initAuthInfo(userId,authInfo);
                }
            }
        }
        MdaEnv.setAuthInfo(authInfo);
        return authInfo;
    }

    private void initAuthInfo(String logName,IAuthInfo authInfo){
//        Map<String,Object> argsMap = new HashMap<>();
//        argsMap.put("sql","select * from usr where extkey='"+logName+"'");
//        List<Usr> usrList = usrMapper.dyncRead(argsMap);
//        Usr usrModule = usrList.get(0);
//        Datas.saveData(usrModule, authInfo.getKeepAuthInfo(),"USR");
//        String ssninr = usrModule.getSsninr();
//        Ssn ssn = new Ssn();
//        ssn.setInr(ssninr);
//        ssn = ssnMapper.selectByPrimaryKey(ssn);
//        Datas.saveData(ssn,authInfo.getKeepAuthInfo(),"SSN");
//
//        argsMap.put("sql","select * from ety where extkey='"+usrModule.getEty()+"'");
//        List<Ety> etyList = etyMapper.dyncRead(argsMap);
//        Ety etyModule = etyList.get(0);
//        Datas.saveData(etyModule,authInfo.getKeepAuthInfo(),"ETY");
//
//
//        argsMap.put("sql","select * from etg where extkey='"+etyModule.getEtg()+"'");
//        List<Etg> etgList = etgMapper.dyncRead(argsMap);
//        Etg etgModule = etgList.get(0);
//        Datas.saveData(etgModule,authInfo.getKeepAuthInfo(),"ETG");
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {

        String logName = request.getHeader("userId");
        if(logName == null || "undefined".equals(logName))
            logName = "sun123";
        String token = getTokenKey(request);
        //目前采用自动登录模式认证
        getOrCreateAuthInfo(token,logName);
        //设置一些系统参数
        String actionTime = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date());
        Systems.setContext(tdContextLASTUSERACTIONTIME,actionTime);
        return true;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {

    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {
        // 请求完成后，需把 authInfo、context 回写到 redis 中，下次请求的会用到。
        // IAuthInfo authInfo = MdaEnv.getAuthInfo();
        // authInfoICache.store(request.getHeader("token"), authInfo);

        IContext context = MdaEnv.getContext();
        if (context != null) {
            BaseVO baseVO = context.getVo();
            String pageId = baseVO.getPageId();
            if (StringUtils.isNotBlank(pageId)) {
                contextICache.store(pageId, context);
            }
        }

        MdaEnv.removeAuthInfo();
        MdaEnv.removeAttrs();
        MdaEnv.clearContext();
        MdaEnv.removeGridData();
    }
}
